package cz.cvut.fel.pr2.telefonniseznam.data;

import cz.cvut.fel.pr2.telefonniseznam.validation.ValidationResult;
import java.text.Collator;
import java.util.Locale;


/**
 *
 * @author balikm1
 */
public class Kontakt implements Comparable<Kontakt> {

    private String jmeno;
    private String prijmeni;
    private String telefonniCislo;

    //<editor-fold defaultstate="collapsed" desc="Initialization">
    /**
     * Constructs a newly allocated Kontakt object that represents a record in the phone book.
     * @param jmeno name of the person
     * @param prijmeni surname of the person
     * @param telefonniCislo phone number of the person
     */
    public Kontakt(String jmeno, String prijmeni, String telefonniCislo)
    {
        ValidationResult result = new ValidationResult();
        validateName(jmeno, result);
        validateSurname(prijmeni, result);
        validatePhone(telefonniCislo, result).throwEx();
        this.jmeno = jmeno;
        this.prijmeni = prijmeni;
        this.telefonniCislo = telefonniCislo;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="Properties">
    public String getJmeno() {
        return jmeno;
    }

    public void setJmeno(String jmeno) {
        validateName(jmeno, new ValidationResult()).throwEx();
        this.jmeno = jmeno;
    }

    public String getPrijmeni() {
        return prijmeni;
    }

    public void setPrijmeni(String prijmeni) {
        validateSurname(prijmeni, new ValidationResult()).throwEx();
        this.prijmeni = prijmeni;
    }

    public String getTelefonniCislo() {
        return telefonniCislo;
    }

    public void setTelefonniCislo(String telefonniCislo) {
        validatePhone(telefonniCislo, new ValidationResult()).throwEx();
        this.telefonniCislo = telefonniCislo;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="Validation methods">
    private ValidationResult validateName(String name, ValidationResult validationResult) {
        if (!assertNotEmpty(name, "Name", validationResult)) return validationResult;
        return validationResult.addCheck(name.chars().allMatch(c -> Character.isLetter((char)c)), "Name can contain only letters.");
    }
    
    private ValidationResult validateSurname(String surname, ValidationResult validationResult) {
        if (!assertNotEmpty(surname, "Surname", validationResult)) return validationResult;
        return validationResult.addCheck(surname.chars().allMatch(c -> Character.isLetter((char)c)), "Surname can contain only letters.");
    }
    
    private ValidationResult validatePhone(String phone, ValidationResult validationResult) {
        if (!assertNotEmpty(phone, "Phone", validationResult)) return validationResult;
        validationResult.addCheck(phone.charAt(0) == '+' || phone.chars().allMatch(c -> Character.isDigit((char)c)) && phone.length() == 9, "Phone needs to have 9 digits.");
        return validationResult.addCheck(phone.charAt(0) != '+' || phone.length() > 4 && phone.substring(0, 3).equals("+420") && phone.substring(4).chars().allMatch(c -> Character.isLetter((char)c)) && phone.substring(4).length() == 9, "Phone needs to be entered in format +420123456789.");
    }
    
    private boolean assertNotEmpty(String value, String fieldName, ValidationResult validationResult) {
        boolean isNotEmpty = value != null && !value.trim().isEmpty();
        validationResult.addCheck(isNotEmpty, String.format("%s can not be empty.", fieldName));
        return isNotEmpty;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="Comparable implementation">
    @Override
    public int compareTo(Kontakt o) {
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;

        if ( this == o ) return EQUAL;

        //Ziskame Collator pro cestinu a nastavime silu na PRIMARY
        Collator csCollator = Collator.getInstance(new Locale("cs", "CZ"));
        csCollator.setStrength(Collator.PRIMARY);

        if (o != null && csCollator.compare(this.prijmeni, o.prijmeni) == EQUAL)
        {
            return csCollator.compare(this.jmeno,o.jmeno);
        }
        else
        {
            return csCollator.compare(this.prijmeni, o.prijmeni);
        }
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="Object overrides">
    @Override
    public boolean equals(Object o)
    {
        if (this == o) return true;

        if (o instanceof Kontakt)
        {
            Kontakt k = (Kontakt)o;
            if (this.jmeno.equals(k.jmeno) && this.prijmeni.equals(k.prijmeni) && this.telefonniCislo.equals(k.telefonniCislo))
                return true;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + (this.jmeno != null ? this.jmeno.hashCode() : 0);
        hash = 59 * hash + (this.prijmeni != null ? this.prijmeni.hashCode() : 0);
        hash = 59 * hash + (this.telefonniCislo != null ? this.telefonniCislo.hashCode() : 0);
        return hash;
    }
    
    @Override
    public String toString()
    {
        return prijmeni + " " + jmeno + " \t" + telefonniCislo;
    }
    //</editor-fold>
}
